@@ -2,30 +2,31 @@
 # this repository contains the full copyright notices and license terms.
 
 
-from itertools import groupby
+import datetime
 from collections import defaultdict
 from decimal import Decimal
-import datetime
+from itertools import groupby
 
 from sql import Null
 from sql.aggregate import Sum
+from sql.conditionals import NullIf
 from sql.functions import Extract
 from sql.operators import Concat
-from sql.conditionals import NullIf
 
 from trytond.i18n import gettext
 from trytond.model import ModelSQL, ModelView, fields
-from trytond.pool import PoolMeta
-from trytond.pyson import Eval, Bool, If, Id, PYSONEncoder
-from trytond.pool import Pool
+from trytond.modules.currency.fields import Monetary
+from trytond.pool import Pool, PoolMeta
+from trytond.pyson import Bool, Eval, Id, If
+from trytond.tools import grouped_slice, reduce_ids
 from trytond.transaction import Transaction
-from trytond.wizard import Wizard, StateAction
-from trytond.tools import reduce_ids, grouped_slice
+from trytond.wizard import StateAction, Wizard
 
 from .exceptions import InvoicingError
 
 
 class Effort:
+    __slots__ = ()
 
     @classmethod
     def __setup__(cls):
@@ -90,7 +91,7 @@ class Effort:
 
 
 class Progress:
-
+    __slots__ = ()
     invoiced_progress = fields.One2Many('project.work.invoiced_progress',
         'work', 'Invoiced Progress', readonly=True)
 
@@ -205,18 +206,9 @@ class Progress:
             origins.append(invoiced_progress)
         return origins
 
-    @classmethod
-    def copy(cls, records, default=None):
-        if default is None:
-            default = {}
-        else:
-            default = default.copy()
-        default.setdefault('invoiced_progress', None)
-        return super().copy(records, default=default)
-
 
 class Timesheet:
-
+    __slots__ = ()
     project_invoice_timesheet_up_to = fields.Date(
         "Invoice up to",
         states={
@@ -240,7 +232,7 @@ class Timesheet:
                 ()),
             ]
         if 'invoice_method' not in cls.product.depends:
-            cls.product.depends.append('invoice_method')
+            cls.product.depends.add('invoice_method')
 
     @fields.depends('type', 'project_invoice_timesheet_up_to',
         'parent', '_parent_parent.invoice_timesheet_up_to')
@@ -377,19 +369,19 @@ class Work(Effort, Progress, Timesheet, metaclass=PoolMeta):
         'on_change_with_invoice_method')
     quantity_to_invoice = fields.Function(
         fields.Float("Quantity to Invoice"), '_get_invoice_values')
-    amount_to_invoice = fields.Function(fields.Numeric("Amount to Invoice",
-            digits=(16, Eval('currency_digits', 2)),
+    amount_to_invoice = fields.Function(Monetary(
+            "Amount to Invoice", currency='currency', digits='currency',
             states={
                 'invisible': Eval('invoice_method') == 'manual',
                 },
-            depends=['currency_digits', 'invoice_method']),
+            depends=['invoice_method']),
         'get_total')
-    invoiced_amount = fields.Function(fields.Numeric('Invoiced Amount',
-            digits=(16, Eval('currency_digits', 2)),
+    invoiced_amount = fields.Function(Monetary(
+            "Invoiced Amount", currency='currency', digits='currency',
             states={
                 'invisible': Eval('invoice_method') == 'manual',
                 },
-            depends=['currency_digits', 'invoice_method']),
+            depends=['invoice_method']),
         'get_total')
     invoice_line = fields.Many2One('account.invoice.line', 'Invoice Line',
         readonly=True)
@@ -558,54 +550,30 @@ class Work(Effort, Progress, Timesheet, metaclass=PoolMeta):
         "Return a invoice line for the lines"
         pool = Pool()
         InvoiceLine = pool.get('account.invoice.line')
-        AccountConfiguration = pool.get('account.configuration')
-        account_config = AccountConfiguration(1)
 
         quantity = sum(l['quantity'] for l in lines)
         product = key['product']
 
-        invoice_line = InvoiceLine()
+        invoice_line = InvoiceLine(invoice=invoice)
+        invoice_line.on_change_invoice()
         invoice_line.type = 'line'
         invoice_line.description = key['description']
-        if product:
-            invoice_line.account = product.account_revenue_used
-            if not invoice_line.account:
+        invoice_line.unit_price = key['unit_price']
+        invoice_line.quantity = quantity
+        invoice_line.unit = key['unit']
+        invoice_line.product = product
+        invoice_line.on_change_product()
+        if not invoice_line.account:
+            if invoice_line.product:
                 raise InvoicingError(
                     gettext(
                         'project_invoice.msg_product_missing_account_revenue',
                         work=self.rec_name,
-                        product=product.rec_name))
-        else:
-            invoice_line.account = account_config.get_multivalue(
-                'default_category_account_revenue', company=invoice.company.id)
-            if not invoice_line.account:
+                        product=invoice_line.product.rec_name))
+            else:
                 raise InvoicingError(
                     gettext('project_invoice.msg_missing_account_revenue',
                         work=self.rec_name))
-        invoice_line.product = product
-        invoice_line.unit_price = key['unit_price']
-        invoice_line.quantity = quantity
-        invoice_line.unit = key['unit']
-        invoice_line.currency = invoice.currency
-        invoice_line.company = invoice.company
-
-        taxes = []
-        pattern = invoice_line._get_tax_rule_pattern()
-        party = invoice.party
-        original_taxes = (
-            product.customer_taxes_used if product else invoice.account.taxes)
-        for tax in original_taxes:
-            if party.customer_tax_rule:
-                tax_ids = party.customer_tax_rule.apply(tax, pattern)
-                if tax_ids:
-                    taxes.extend(tax_ids)
-                continue
-            taxes.append(tax.id)
-        if party.customer_tax_rule:
-            tax_ids = party.customer_tax_rule.apply(None, pattern)
-            if tax_ids:
-                taxes.extend(tax_ids)
-        invoice_line.taxes = taxes
         return invoice_line
 
     def _test_group_invoice(self):
@@ -721,8 +689,4 @@ class OpenInvoice(Wizard):
             if work.invoiced_progress:
                 for progress in work.invoiced_progress:
                     invoice_ids.add(progress.invoice_line.invoice.id)
-        encoder = PYSONEncoder()
-        action['pyson_domain'] = encoder.encode(
-            [('id', 'in', list(invoice_ids))])
-        action['pyson_search_value'] = encoder.encode([])
-        return action, {}
+        return action, {'res_id': list(invoice_ids)}
